@@ -32,6 +32,7 @@
 #include "nautilus-file-utilities.h"
 #include "nautilus-global-preferences.h"
 #include "nautilus-link.h"
+#include "nautilus-marshal.h"
 #include "nautilus-metafile.h"
 #include "nautilus-search-uri.h"
 #include <eel/eel-glib-extensions.h>
@@ -94,6 +95,13 @@ typedef struct {
 	Request request;
 } Monitor;
 
+typedef struct {
+	NautilusDirectory *directory;
+	NautilusInfoProvider *provider;
+	NautilusOperationHandle *handle;
+	NautilusOperationResult result;
+} InfoProviderResponse;
+
 typedef gboolean (* RequestCheck) (const Request *);
 typedef gboolean (* FileCheck) (NautilusFile *);
 
@@ -127,8 +135,12 @@ static gboolean file_needs_high_priority_work_done (NautilusDirectory *directory
 						    NautilusFile      *file);
 static gboolean file_needs_low_priority_work_done  (NautilusDirectory *directory,
 						    NautilusFile      *file);
+static gboolean file_needs_extension_work_done  (NautilusDirectory *directory,
+						 NautilusFile      *file);
 static void     move_file_to_low_priority_queue    (NautilusDirectory *directory,
 						    NautilusFile      *file);
+static void     move_file_to_extension_queue    (NautilusDirectory *directory,
+						 NautilusFile      *file);
 
 
 void
@@ -529,6 +541,7 @@ nautilus_directory_set_up_request (Request *request,
 			      NAUTILUS_FILE_ATTRIBUTE_DIRECTORY_ITEM_MIME_TYPES) != FALSE;
 	request->file_info = (file_attributes & 
 			      (NAUTILUS_FILE_ATTRIBUTE_MIME_TYPE |
+			       NAUTILUS_FILE_ATTRIBUTE_SLOW_MIME_TYPE |
 			       NAUTILUS_FILE_ATTRIBUTE_IS_DIRECTORY |
 			       NAUTILUS_FILE_ATTRIBUTE_CAPABILITIES |
 			       NAUTILUS_FILE_ATTRIBUTE_FILE_TYPE)) != FALSE;
@@ -571,6 +584,10 @@ nautilus_directory_set_up_request (Request *request,
 	
 	request->metafile |= (file_attributes & 
 			      NAUTILUS_FILE_ATTRIBUTE_METADATA) != FALSE;
+
+	request->slow_mime_type = (file_attributes & NAUTILUS_FILE_ATTRIBUTE_SLOW_MIME_TYPE) != FALSE;
+
+	request->extension_info = (file_attributes & NAUTILUS_FILE_ATTRIBUTE_EXTENSION_INFO) != FALSE;
 }
 
 static void
@@ -588,6 +605,7 @@ mime_db_changed_callback (GnomeVFSMIMEMonitor *ignore, NautilusDirectory *dir)
 		NAUTILUS_FILE_ATTRIBUTE_CAPABILITIES |
 		NAUTILUS_FILE_ATTRIBUTE_CUSTOM_ICON |
 		NAUTILUS_FILE_ATTRIBUTE_MIME_TYPE |
+		NAUTILUS_FILE_ATTRIBUTE_SLOW_MIME_TYPE |
 		NAUTILUS_FILE_ATTRIBUTE_METADATA |
 		NAUTILUS_FILE_ATTRIBUTE_FILE_TYPE |
 		NAUTILUS_FILE_ATTRIBUTE_DIRECTORY_ITEM_MIME_TYPES;
@@ -880,7 +898,7 @@ dequeue_pending_idle_callback (gpointer callback_data)
 		if (file != NULL) {
 			/* file already exists, check if it changed */
 			set_file_unconfirmed (file, FALSE);
-			if (nautilus_file_update_info (file, file_info)) {
+			if (nautilus_file_update_info (file, file_info, FALSE)) {
 				/* File changed, notify about the change. */
 				nautilus_file_ref (file);
 				changed_files = g_list_prepend (changed_files, file);
@@ -1263,9 +1281,11 @@ ready_callback_call (NautilusDirectory *directory,
 
 	/* Call the callback. */
 	if (callback->file != NULL) {
-		(* callback->callback.file) (callback->file,
-					     callback->callback_data);
-	} else {
+		if (callback->callback.file) {
+			(* callback->callback.file) (callback->file,
+						     callback->callback_data);
+		}
+	} else if (callback->callback.directory != NULL) {
 		if (directory == NULL || !callback->request.file_list) {
 			file_list = NULL;
 		} else {
@@ -1295,7 +1315,6 @@ nautilus_directory_call_when_ready_internal (NautilusDirectory *directory,
 	g_assert (directory == NULL || NAUTILUS_IS_DIRECTORY (directory));
 	g_assert (file == NULL || NAUTILUS_IS_FILE (file));
 	g_assert (file != NULL || directory_callback != NULL);
-	g_assert (file == NULL || file_callback != NULL);
 	
 	/* Construct a callback object. */
 	callback.file = file;
@@ -1307,7 +1326,7 @@ nautilus_directory_call_when_ready_internal (NautilusDirectory *directory,
 	callback.callback_data = callback_data;
 	nautilus_directory_set_up_request (&callback.request, file_attributes);
 	callback.request.file_list = wait_for_file_list;
-	
+
 	/* Handle the NULL case. */
 	if (directory == NULL) {
 		ready_callback_call (NULL, &callback);
@@ -1318,7 +1337,10 @@ nautilus_directory_call_when_ready_internal (NautilusDirectory *directory,
 	if (g_list_find_custom (directory->details->call_when_ready_list,
 				&callback,
 				ready_callback_key_compare) != NULL) {
-		g_warning ("tried to add a new callback while an old one was pending");
+		if (file_callback != NULL && directory_callback != NULL) {
+			g_warning ("tried to add a new callback while an old one was pending");
+		}
+		/* NULL callback means, just read it. Conflicts are ok. */
 		return;
 	}
 
@@ -1655,6 +1677,26 @@ wants_info (const Request *request)
 	return request->file_info;
 }
 
+static gboolean
+always_lacks (NautilusFile *file)
+{
+	return TRUE;
+}
+
+static gboolean
+lacks_slow_mime_type (NautilusFile *file)
+{
+	return !file->details->got_slow_mime_type
+		&& !file->details->is_gone;
+}
+
+static gboolean
+wants_slow_mime_type (const Request *request)
+{
+	return request->slow_mime_type;
+}
+
+
 static gboolean
 lacks_deep_count (NautilusFile *file)
 {
@@ -1748,6 +1790,17 @@ wants_link_info (const Request *request)
 	return request->link_info;
 }
 
+static gboolean
+lacks_extension_info (NautilusFile *file)
+{
+	return file->details->pending_info_providers != NULL;
+}
+
+static gboolean
+wants_extension_info (const Request *request)
+{
+	return request->extension_info;
+}
 
 static gboolean
 has_problem (NautilusDirectory *directory, NautilusFile *file, FileCheck problem)
@@ -1793,6 +1846,12 @@ request_is_satisfied (NautilusDirectory *directory,
 		}
 	}
 
+	if (request->slow_mime_type) {
+		if (has_problem (directory, file, lacks_slow_mime_type)) {
+			return FALSE;
+		}
+	}
+
 	if (request->top_left_text) {
 		if (has_problem (directory, file, lacks_top_left)) {
 			return FALSE;
@@ -2072,7 +2131,7 @@ start_monitoring_file_list (NautilusDirectory *directory)
 	
 #ifdef DEBUG_LOAD_DIRECTORY
 	g_message ("load_directory called to monitor file list of %s", directory->details->uri);
-#endif	
+#endif
 	gnome_vfs_async_load_directory
 		(&directory->details->directory_load_in_progress, /* handle */
 		 directory->details->uri,                         /* uri */
@@ -2766,6 +2825,7 @@ get_info_callback (GnomeVFSAsyncHandle *handle,
 	NautilusDirectory *directory;
 	NautilusFile *get_info_file;
 	GnomeVFSGetFileInfoResult *result;
+	gboolean has_slow_mime_type;
 
 	directory = NAUTILUS_DIRECTORY (callback_data);
 	g_assert (handle == NULL || handle == directory->details->get_info_in_progress);
@@ -2774,10 +2834,13 @@ get_info_callback (GnomeVFSAsyncHandle *handle,
 	g_assert (NAUTILUS_IS_FILE (get_info_file));
 
 	nautilus_directory_ref (directory);
+
+	has_slow_mime_type = directory->details->get_info_has_slow_mime_type;
 	
 	directory->details->get_info_file = NULL;
 	directory->details->get_info_in_progress = NULL;
-
+	directory->details->get_info_has_slow_mime_type = FALSE;
+	
 	/* ref here because we might be removing the last ref when we
 	 * mark the file gone below, but we need to keep a ref at
 	 * least long enough to send the change notification. 
@@ -2803,7 +2866,7 @@ get_info_callback (GnomeVFSAsyncHandle *handle,
 			}
 		}
 	} else {
-		nautilus_file_update_info (get_info_file, result->file_info);
+		nautilus_file_update_info (get_info_file, result->file_info, has_slow_mime_type);
 	}
 
 	nautilus_file_changed (get_info_file);
@@ -2825,7 +2888,8 @@ file_info_stop (NautilusDirectory *directory)
 		if (file != NULL) {
 			g_assert (NAUTILUS_IS_FILE (file));
 			g_assert (file->details->directory == directory);
-			if (is_needy (file, lacks_info, wants_info)) {
+			if (is_needy (file, lacks_info, wants_info) ||
+			    is_needy (file, lacks_slow_mime_type, wants_slow_mime_type)) {
 				return;
 			}
 		}
@@ -2842,16 +2906,20 @@ file_info_start (NautilusDirectory *directory,
 	char *uri;
 	GnomeVFSURI *vfs_uri;
 	GList fake_list;
-
+	gboolean need_slow_mime;
+	GnomeVFSFileInfoOptions options;
+	
 	file_info_stop (directory);
 
 	if (directory->details->get_info_in_progress != NULL) {
 		return;
 	}
 
-	if (!is_needy (file, lacks_info, wants_info)) {
+	if (!is_needy (file, lacks_info, wants_info) &&
+	    !is_needy (file, lacks_slow_mime_type, wants_slow_mime_type)) {
 		return;
 	}
+	need_slow_mime = is_needy (file, always_lacks, wants_slow_mime_type);
 	
 	uri = nautilus_file_get_uri (file);
 	vfs_uri = gnome_vfs_uri_new (uri);
@@ -2864,6 +2932,7 @@ file_info_start (NautilusDirectory *directory,
 		file->details->file_info_is_up_to_date = TRUE;
 		file->details->get_info_failed = TRUE;
 		file->details->get_info_error = GNOME_VFS_ERROR_INVALID_URI;
+		file->details->got_slow_mime_type = need_slow_mime;
 
 		nautilus_directory_async_state_changed (directory);
 		return;
@@ -2877,18 +2946,23 @@ file_info_start (NautilusDirectory *directory,
 	fake_list.data = vfs_uri;
 	fake_list.prev = NULL;
 	fake_list.next = NULL;
+
+	options = GNOME_VFS_FILE_INFO_GET_MIME_TYPE
+		| GNOME_VFS_FILE_INFO_FOLLOW_LINKS;
+	if (need_slow_mime) {
+		options |= GNOME_VFS_FILE_INFO_FORCE_SLOW_MIME_TYPE;
+	}
+	directory->details->get_info_has_slow_mime_type = need_slow_mime;
 	gnome_vfs_async_get_file_info
 		(&directory->details->get_info_in_progress,
 		 &fake_list,
-		 GNOME_VFS_FILE_INFO_GET_MIME_TYPE
-		 | GNOME_VFS_FILE_INFO_FOLLOW_LINKS,
+		 options,
 		 GNOME_VFS_PRIORITY_DEFAULT,
 		 get_info_callback,
 		 directory);
 	gnome_vfs_uri_unref (vfs_uri);
 }
 
-
 static void
 link_info_done (NautilusDirectory *directory,
 		NautilusFile *file,
@@ -3135,6 +3209,153 @@ link_info_start (NautilusDirectory *directory,
 	g_free (uri);
 }
 
+static void
+extension_info_cancel (NautilusDirectory *directory)
+{
+	if (directory->details->extension_info_in_progress != NULL) {
+		nautilus_info_provider_cancel_update 
+			(directory->details->extension_info_provider,
+			 directory->details->extension_info_in_progress);
+
+		directory->details->extension_info_in_progress = NULL;
+		directory->details->extension_info_file = NULL;
+		directory->details->extension_info_provider = NULL;
+
+		async_job_end (directory, "extension info");
+	}
+}
+	
+static void
+extension_info_stop (NautilusDirectory *directory)
+{
+	if (directory->details->extension_info_in_progress != NULL) {
+		NautilusFile *file;
+
+		file = directory->details->extension_info_file;
+		if (file != NULL) {
+			g_assert (NAUTILUS_IS_FILE (file));
+			g_assert (file->details->directory == directory);
+			if (is_needy (file, lacks_extension_info, wants_extension_info)) {
+				return;
+			}
+		}
+
+		/* The info is not wanted, so stop it. */
+		extension_info_cancel (directory);
+	}
+}
+
+static void
+finish_info_provider (NautilusDirectory *directory,
+		      NautilusFile *file,
+		      NautilusInfoProvider *provider)
+{
+	file->details->pending_info_providers = 
+		g_list_remove  (file->details->pending_info_providers,
+				provider);
+	g_object_unref (provider);
+
+	nautilus_directory_async_state_changed (directory);
+
+	if (file->details->pending_info_providers == NULL) {
+		nautilus_file_info_providers_done (file);
+	}
+}
+
+
+static gboolean
+info_provider_idle_callback (gpointer user_data)
+{
+	InfoProviderResponse *response;
+	NautilusDirectory *directory;
+
+	response = user_data;
+	directory = response->directory;
+
+	if (response->handle != directory->details->extension_info_in_progress
+	    || response->provider != directory->details->extension_info_provider) {
+		g_warning ("Unexpected plugin response.  This probably indicates a bug in a Nautilus extension: handle=%p", response->handle);
+	} else {
+		NautilusFile *file;
+		async_job_end (directory, "extension info");
+
+		file = directory->details->extension_info_file;
+
+		directory->details->extension_info_file = NULL;
+		directory->details->extension_info_provider = NULL;
+		directory->details->extension_info_in_progress = NULL;
+
+		finish_info_provider (directory, file, response->provider);
+	}
+
+	return FALSE;
+}
+
+static void
+info_provider_callback (NautilusInfoProvider *provider,
+			NautilusOperationHandle *handle,
+			NautilusOperationResult result,
+			gpointer user_data)
+{
+	InfoProviderResponse *response;
+	
+	response = g_new0 (InfoProviderResponse, 1);
+	response->provider = provider;
+	response->handle = handle;
+	response->result = result;
+	response->directory = NAUTILUS_DIRECTORY (user_data);
+
+	g_idle_add (info_provider_idle_callback, response);
+}
+
+static void
+extension_info_start (NautilusDirectory *directory,
+		      NautilusFile *file)
+{
+	NautilusInfoProvider *provider;
+	NautilusOperationResult result;
+	NautilusOperationHandle *handle;
+	GClosure *update_complete;
+
+	if (directory->details->extension_info_in_progress != NULL) {
+		return;
+	}
+	
+	if (!is_needy (file, lacks_extension_info, wants_extension_info)) {
+		return;
+	}
+
+	if (!async_job_start (directory, "extension info")) {
+		return;
+	}
+
+	provider = file->details->pending_info_providers->data;
+
+	update_complete = g_cclosure_new (G_CALLBACK (info_provider_callback),
+					  directory,
+					  NULL);
+	g_closure_set_marshal (update_complete,
+			       nautilus_marshal_VOID__POINTER_ENUM);
+			       
+	result = nautilus_info_provider_update_file_info
+		(provider, 
+		 NAUTILUS_FILE_INFO (file), 
+		 update_complete, 
+		 &handle);
+
+	g_closure_unref (update_complete);
+
+	if (result == NAUTILUS_OPERATION_COMPLETE ||
+	    result == NAUTILUS_OPERATION_FAILED) {
+		finish_info_provider (directory, file, provider);
+		async_job_end (directory, "extension info");
+	} else {
+		directory->details->extension_info_in_progress = handle;
+		directory->details->extension_info_provider = provider;
+		directory->details->extension_info_file = file;
+	}
+}
+
 static void
 start_or_stop_io (NautilusDirectory *directory)
 {
@@ -3150,6 +3371,7 @@ start_or_stop_io (NautilusDirectory *directory)
 	mime_list_stop (directory);
 	top_left_stop (directory);
 	link_info_stop (directory);
+	extension_info_stop (directory);
 
 	/* Take files that are all done off the queue. */
 	while (!nautilus_file_queue_is_empty (directory->details->high_priority_queue)) {
@@ -3171,11 +3393,26 @@ start_or_stop_io (NautilusDirectory *directory)
 
 		if (file_needs_low_priority_work_done (directory, file)) {
 			/* Start getting attributes if possible */
+			file_info_start (directory, file); /* for slow mime type */
 			directory_count_start (directory, file);
 			deep_count_start (directory, file);
 			mime_list_start (directory, file);
 			top_left_start (directory, file);
 			return;
+		} else {
+			move_file_to_extension_queue (directory, file);
+
+		}
+	}
+
+	/* Low priority queue must be empty */
+	while (!nautilus_file_queue_is_empty (directory->details->extension_queue)) {
+		file = nautilus_file_queue_head (directory->details->extension_queue);
+
+		if (file_needs_extension_work_done (directory, file)) {
+			/* Start getting attributes if possible */
+			extension_info_start (directory, file);
+			return;
 		} else {
 			nautilus_directory_remove_file_from_work_queue (directory, file);
 
@@ -3228,6 +3465,7 @@ nautilus_directory_cancel (NautilusDirectory *directory)
 	link_info_cancel (directory);
 	mime_list_cancel (directory);
 	top_left_cancel (directory);
+	extension_info_cancel (directory);
 
 	/* We aren't waiting for anything any more. */
 	if (waiting_directories != NULL) {
@@ -3322,6 +3560,10 @@ cancel_loading_attributes (NautilusDirectory *directory,
 	if (request.link_info) {
 		link_info_cancel (directory);
 	}
+
+	if (request.extension_info) {
+		extension_info_cancel (directory);
+	}
 	
 	/* FIXME bugzilla.gnome.org 45064: implement cancelling metadata when we
 	   implement invalidating metadata */
@@ -3399,6 +3641,10 @@ file_needs_low_priority_work_done (NautilusDirectory *directory,
 		return TRUE;
 	}
 
+	if (is_needy (file, lacks_slow_mime_type, wants_slow_mime_type)) {
+		return TRUE;
+	}
+
 	if (is_needy (file, lacks_top_left, wants_top_left)) {
 		return TRUE;
 	}
@@ -3406,12 +3652,24 @@ file_needs_low_priority_work_done (NautilusDirectory *directory,
 	return FALSE;
 }
 
+static gboolean
+file_needs_extension_work_done (NautilusDirectory *directory,
+				   NautilusFile *file)
+{
+	if (is_needy (file, lacks_extension_info, wants_extension_info)) {
+		return TRUE;
+	}
+
+	return FALSE;
+}
+
 static gboolean
 file_needs_work_done (NautilusDirectory *directory,
 		      NautilusFile *file)
 {
 	return (file_needs_high_priority_work_done (directory, file) ||
-		file_needs_low_priority_work_done (directory, file));
+		file_needs_low_priority_work_done (directory, file) || 
+		file_needs_extension_work_done (directory, file));
 }
 
 
@@ -3451,6 +3709,8 @@ nautilus_directory_remove_file_from_work_queue (NautilusDirectory *directory,
 				    file);
 	nautilus_file_queue_remove (directory->details->low_priority_queue,
 				    file);
+	nautilus_file_queue_remove (directory->details->extension_queue,
+				    file);
 }
 
 
@@ -3458,15 +3718,32 @@ static void
 move_file_to_low_priority_queue (NautilusDirectory *directory,
 				 NautilusFile *file)
 {
-	if (!file_needs_low_priority_work_done (directory, file)) {
-		nautilus_file_queue_remove (directory->details->high_priority_queue,
-					    file);
-		return;
-	}
-
 	/* Must add before removing to avoid ref underflow */
 	nautilus_file_queue_enqueue (directory->details->low_priority_queue,
 				     file);
 	nautilus_file_queue_remove (directory->details->high_priority_queue,
 				    file);
+
+	if (!file_needs_low_priority_work_done (directory, file)) {
+		move_file_to_extension_queue (directory, file);
+
+		return;
+	}
+}
+
+static void
+move_file_to_extension_queue (NautilusDirectory *directory,
+			      NautilusFile *file)
+{
+	if (!file_needs_extension_work_done (directory, file)) {
+		nautilus_file_queue_remove (directory->details->low_priority_queue,
+					    file);
+		return;
+	}
+
+	/* Must add before removing to avoid ref underflow */
+	nautilus_file_queue_enqueue (directory->details->extension_queue,
+				     file);
+	nautilus_file_queue_remove (directory->details->low_priority_queue,
+				    file);
 }
