@@ -98,7 +98,8 @@ nautilus_mime_actions_get_minimum_file_attributes (void)
 	return NAUTILUS_FILE_ATTRIBUTE_VOLUMES |
 		NAUTILUS_FILE_ATTRIBUTE_ACTIVATION_URI |
 		NAUTILUS_FILE_ATTRIBUTE_METADATA |
-		NAUTILUS_FILE_ATTRIBUTE_MIME_TYPE;
+		NAUTILUS_FILE_ATTRIBUTE_MIME_TYPE |
+		NAUTILUS_FILE_ATTRIBUTE_SLOW_MIME_TYPE;
 }
 
 
