@@ -993,8 +993,8 @@ remove_from_dialog (FMPropertiesWindow *window,
 					      G_CALLBACK (properties_window_update),
 					      window);
 
-	nautilus_file_monitor_remove (original_file, window);
-	nautilus_file_monitor_remove (target_file, window);
+	nautilus_file_monitor_remove (original_file, &window->details->original_files);
+	nautilus_file_monitor_remove (target_file, &window->details->target_files);
 
 	nautilus_file_unref (original_file);
 	nautilus_file_unref (target_file);
@@ -1756,13 +1756,10 @@ directory_contents_value_field_update (FMPropertiesWindow *window)
 	} else {
 		char *size_str;
 		size_str = gnome_vfs_format_file_size_for_display (total_size);
-		if (total_count == 1) {
-			text = g_strdup_printf (_("1 item, with size %s"),
-						size_str);
-		} else {
-			text = g_strdup_printf (_("%d items, totalling %s"),
-						total_count, size_str);
-		}
+		text = g_strdup_printf (ngettext("%d item, with size %s",
+						 "%d items, totalling %s",
+						 total_count),
+					total_count, size_str);
 		g_free (size_str);
 
 		if (unreadable_directory_count != 0) {
@@ -3227,10 +3224,11 @@ create_properties_window (StartupData *startup_data)
 		file = NAUTILUS_FILE (l->data);
 
 		attributes = nautilus_icon_factory_get_required_file_attributes ();
-		attributes |= NAUTILUS_FILE_ATTRIBUTE_DISPLAY_NAME;
+		attributes |= NAUTILUS_FILE_ATTRIBUTE_DISPLAY_NAME
+			| NAUTILUS_FILE_ATTRIBUTE_SLOW_MIME_TYPE;
 
 		nautilus_file_monitor_add (NAUTILUS_FILE (l->data),
-					   window, 
+					   &window->details->original_files, 
 					   attributes);	
 	}
 	
@@ -3246,7 +3244,7 @@ create_properties_window (StartupData *startup_data)
 		}
 		
 		attributes |= NAUTILUS_FILE_ATTRIBUTE_METADATA;
-		nautilus_file_monitor_add (file, window, attributes);
+		nautilus_file_monitor_add (file, &window->details->target_files, attributes);
 	}	
 		
 	for (l = window->details->target_files; l != NULL; l = l->next) {
@@ -3548,13 +3546,13 @@ real_destroy (GtkObject *object)
 	remove_window (window);
 
 	for (l = window->details->original_files; l != NULL; l = l->next) {
-		nautilus_file_monitor_remove (NAUTILUS_FILE (l->data), window);
+		nautilus_file_monitor_remove (NAUTILUS_FILE (l->data), &window->details->original_files);
 	}
 	nautilus_file_list_free (window->details->original_files);
 	window->details->original_files = NULL;
 	
 	for (l = window->details->target_files; l != NULL; l = l->next) {
-		nautilus_file_monitor_remove (NAUTILUS_FILE (l->data), window);
+		nautilus_file_monitor_remove (NAUTILUS_FILE (l->data), &window->details->target_files);
 	}
 	nautilus_file_list_free (window->details->target_files);
 	window->details->target_files = NULL;
